import tea
import csv
import os
import pandas as pd

def try_something(num): 
    print("Here we are.")
    import pdb; pdb.set_trace()
    print("After PDB!")
    
    if num > 5: 
        pass
    else: 
        print("Less than 5!")

def do_nothing(): 
    pass

if __name__ == "__main__":
    do_nothing()
    # try_something(1)


import tea
variables = [
    {
        'name' : 'So', # Southern 
        'data type': 'nominal',
        'categories': ['0','1']
    },
    {
        'name': 'Prob',
        'data type': 'ratio',
        'range': [0,1]
    }
] 
experimental_design = {
        'study type': 'observational study',
        'contributor variables': 'So',
        'outcome variables': 'Prob'
}
assumptions = {
    'Type 1 (False Positive) Error Rate': 0.05,
}

tea.data('.tea/UScrime.csv')
tea.define_variables(variables)
tea.define_study_design(experimental_design)
tea.assume(assumptions)
tea.hypothesize(['So','Prob'], ['So:1>0']) # Southern is greater



# Test that when the Students t-test is valid, Pointbiserial is as well. 

# When pointbiserial is valid, Students t shuold be, too -- they have the same constraints

# Add test about what happens when have different sizes in samples in pearson r


from build.lib.tea.ast import DataType
import tea.api
from tea.api import define_variables
import unittest
from importlib import reload


class ApiTests(unittest.TestCase):

    def setUp(self):
        reload(tea.api)

    def test_define_variables_should_give_correct_length(self):
        define_variables(DataForTests.variables_to_define)

        # ASSERT
        from tea.api import vars_objs as vars_to_test
        self.assertEqual(len(vars_to_test), 4)

    def test_vars_objs_should_be_empty_before_defining(self):
        from tea.api import vars_objs as vars_to_test
        self.assertEqual(len(vars_to_test), 0)

    def test_define_variables_should_have_correct_names(self):
        expected_names = ['NominalT', 'IntervalT', 'OrdinalT', 'RatioT']

        # ACT
        define_variables(DataForTests.variables_to_define)
        from tea.api import vars_objs as vars_to_test
        real_names = [var.name for var in vars_to_test]

        # ASSERT
        self.assertCountEqual(real_names, expected_names)

    def test_define_variables_should_have_correct_types(self):

        define_variables(DataForTests.variables_to_define)
        from tea.api import vars_objs as vars_to_test
        sorted_vars = sorted(vars_to_test, key=lambda x: x.name)

        self.assertEqual(sorted_vars[0].dtype.value,
                         DataType.INTERVAL.value)  # IntervalT
        self.assertEqual(sorted_vars[1].dtype.value,
                         DataType.NOMINAL.value)  # NominalT
        self.assertEqual(sorted_vars[2].dtype.value,
                         DataType.ORDINAL.value)  # OrdinalT
        self.assertEqual(sorted_vars[3].dtype.value,
                         DataType.RATIO.value)  # RatioT


class DataForTests:

    variables_to_define = [
        {
            'name': 'RatioT',
            'data type': 'ratio'
        },
        {
            'name': 'NominalT',
            'data type': 'nominal',
            'categories': ['Nominal0']
        },
        {
            'name': 'OrdinalT',
            'data type': 'ordinal',
            'categories': ['Ordinal1', 'Ordinal2', 'Ordinal3']
        },
        {
            'name': 'IntervalT',
            'data type': 'interval',
        }
    ]



import numpy as np 


def break_ties(arr, ranks):
    new_ranks = np.copy(ranks)
    assert(isinstance(ranks, np.ndarray))

    counts = np.bincount(ranks)
    assert(counts[0] == 0) # There shouldn't be any elts in ranks with rank 0

    # Go through counts of ranks
    for i in range(counts): 
        # If there are multiple elts with the same rank
        if counts[i] > 1:
            # Find the elt with the rank and break the tie
            for j in range(ranks):
                if ranks[j] == i: 
                    new_ranks[j] = ranks[j]/counts[i] # divide by the number of elts that have that same rank
    
    return new_ranks

def sort_with_ties(arr):
    pass

def rank(arr):

    tmp = arr.argsort() # Returns numpy array of indices ordering their elts
    ranks = np.empty_like(tmp)
    ranks[tmp] = np.arange(len(arr))

    # If there are ties
    if len(set(arr)) != len(arr):
        # Go through and recompute ranks for elts with the same value
        # Get unique elts
        unique_elts = np.unique(arr)

        for e in range(len(unique_elts)):
            idxs = np.where(arr == unique_elts[e])[0] # list of indices for each elt
            # Update ranks
            if len(idxs) > 1: 
                rank_sum = np.sum(ranks[idxs])
                new_rank = rank_sum/len(idxs)
                ranks = ranks.astype('float64') # to allow for new_rank
                ranks[idxs] = new_rank

    return ranks

arr0 = np.array([1, 10, 12, 10,6,8,3])
# arr0 = np.array([10, 10, 1])
ranks = rank(arr0)
import pdb; pdb.set_trace()


import pandas as pd

import tea
import os

base_url = 'https://homes.cs.washington.edu/~emjun/tea-lang/datasets/'
file_names = ['UScrime.csv', 'statex77.csv', 'catsData.csv', 'cholesterol.csv', 'soya.csv', 'co2.csv', 'exam.csv', 'liar.csv',
              'pbcorr.csv', 'spiderLong_within.csv', 'drug.csv', 'alcohol.csv', 'ecstasy.csv', 'gogglesData.csv', 'gogglesData_dummy.csv']
data_paths = [None] * len(file_names)


def load_data():
    global base_url, data_paths, file_names
    global drug_path

    for i in range(len(data_paths)):
        csv_name = file_names[i]

        csv_url = os.path.join(base_url, csv_name)
        data_paths[i] = tea.download_data(csv_url, csv_name)


def get_data_path(filename):
    load_data()
    try:
        data_idx = file_names.index(filename)
    except:
        raise ValueError(f"File is not found!:{filename}")
    data_path = data_paths[data_idx]

    return data_path

# Example from Kabacoff
# Expected outcome: Pearson correlation


def test_pearson_corr():
    data_path = get_data_path('statex77.csv')
    # data_path2 = get_data_path('statex87.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'Illiteracy',
            'data type': 'interval',
            'categories': [0, 100]
        },
        {
            'name': 'Life Exp',
            'data type': 'ratio',
        }
    ]
    experimental_design = {
        'study type': 'observational study',
        'contributor variables': ['Illiteracy', 'Life Exp'],
        'outcome variables': ''
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
        'normal distribution': ['Illiteracy']
    }

    tea.data(data_path)
    # tea.data(data_path2)
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions, 'strict')

    results = tea.hypothesize(['Illiteracy', 'Life Exp'], [
                              'Illiteracy ~ Life Exp'])
    # print("\nfrom Kabacoff")
    # print("Expected outcome: Pearson")
    print('++++++++++++')


def test_pearson_corr_2():
    data_path = get_data_path('exam.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'Exam',
            'data type': 'ratio',
            'range': [0, 100]
        },
        {
            'name': 'Anxiety',
            'data type': 'interval',
            'range': [0, 100]
        },
        {
            'name': 'Gender',
            'data type': 'nominal',
            'categories': ['Male', 'Female']
        },
        {
            'name': 'Revise',
            'data type': 'ratio'
        }
    ]
    experimental_design = {
        'study type': 'observational study',
        'contributor variables': ['Anxiety', 'Gender', 'Revise'],
        'outcome variables': 'Exam'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_path)
    tea.define_variables(variables)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    results = tea.hypothesize(['Anxiety', 'Exam'])
    results = tea.hypothesize(['Revise', 'Exam'])
    results = tea.hypothesize(['Anxiety', 'Revise'])
    # print("\nfrom Field et al.")
    # print("Expected outcome: Pearson")
    print('++++++++++++')


def test_spearman_corr():
    data_path = get_data_path('liar.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'Creativity',
            'data type': 'interval'
        },
        {
            'name': 'Position',
            'data type': 'ordinal',
            'categories': [6, 5, 4, 3, 2, 1]  # ordered from lowest to highest
        },
        {
            'name': 'Novice',
            'data type': 'nominal',
            'categories': [0, 1]
        }
    ]
    experimental_design = {
        'study type': 'observational study',
        'contributor variables': ['Novice', 'Creativity'],
        'outcome variables': 'Position'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_path)
    tea.define_variables(variables)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    # TODO: allow for partial orders?
    results = tea.hypothesize(['Position', 'Creativity'], ['Position:1 > 6'])
    # print("\nfrom Field et al.")
    # print("Expected outcome: Spearman")
    print('++++++++++++')

# Same as test for Spearman rho


def test_kendall_tau_corr():
    data_path = get_data_path('liar.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'Creativity',
            'data type': 'interval'
        },
        {
            'name': 'Position',
            'data type': 'ordinal',
            'categories': [6, 5, 4, 3, 2, 1]  # ordered from lowest to highest
        },
        {
            'name': 'Novice',
            'data type': 'nominal',
            'categories': [0, 1]
        }
    ]
    experimental_design = {
        'study type': 'observational study',
        'contributor variables': ['Novice', 'Creativity'],
        'outcome variables': 'Position'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_path)
    tea.define_variables(variables)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    results = tea.hypothesize(['Position', 'Creativity'], [
                              'Position:1 > 6', 'Position:1 > 2'])  # I think this works!?
    # print("\nfrom Field et al.")
    # print("Expected outcome: Kendall Tau")
    print('++++++++++++')


def test_pointbiserial_corr():
    data_path = get_data_path('pbcorr.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'time',
            'data type': 'ratio'
        },
        {
            'name': 'gender',
            'data type': 'nominal',
            'categories': [0, 1]  # ordered from lowest to highest
        },
        {
            'name': 'recode',
            'data type': 'nominal',
            'categories': [0, 1]
        }
    ]
    experimental_design = {
        'study type': 'observational study',
        'contributor variables': ['gender', 'recode'],
        'outcome variables': 'time'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_path)
    tea.define_variables(variables)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    # I think this works!?
    tea.hypothesize(['time', 'gender'], ['gender:1 > 0'])
    # print("\nfrom Field et al.")
    # print("Expected outcome: Pointbiserial")
    print('++++++++++++')


def test_indep_t_test():
    data_path = get_data_path('UScrime.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'So',
            'data type': 'nominal',
            'categories': ['0', '1']
        },
        {
            'name': 'Prob',
            'data type': 'ratio',
            'range': [0, 1]
        }
    ]
    experimental_design = {
        'study type': 'observational study',
        'contributor variables': 'So',
        'outcome variables': 'Prob',
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
        'groups normally distributed': [['Prob', 'So']]
    }

    transformations = {
        'log': ['Prob']
    }

    tea.data(data_path)
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['So', 'Prob'], ['So:1 > 0'])  # Southern is greater
    # print("\nfrom Kabacoff")
    # print("Expected outcome: Student's t-test")
    print('++++++++++++')


def test_paired_t_test():
    data_path = get_data_path('spiderLong_within.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'Group',
            'data type': 'nominal',
            'categories': ['Picture', 'Real Spider']
        },
        {
            'name': 'Anxiety',
            'data type': 'ratio'
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': 'Group',
        'dependent variables': 'Anxiety',
        'within subjects': 'Group'

    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05
    }

    tea.data(data_path, key="id")
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['Group', 'Anxiety'], ['Group:Real Spider > Picture'])

    # print("\nfrom Field et al.")
    # print("Expected outcome: Paired/Dependent t-test")
    print('++++++++++++')


def test_wilcoxon_signed_rank():
    data_path = get_data_path('alcohol.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'drug',
            'data type': 'nominal',
            'categories': ['Alcohol']
        },
        {
            'name': 'day',
            'data type': 'nominal',
            'categories': ['sundayBDI', 'wedsBDI']
        },
        {
            'name': 'value',
            'data type': 'ratio'
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': 'day',
        'dependent variables': 'value',
        'within subjects': 'day'

    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05
    }

    tea.data(data_path)
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['day', 'value'], ['day:sundayBDI != wedsBDI'])
    
    # print("\nfrom Field et al.")
    # print("Expected outcome: Wilcoxon signed rank test")
    print('++++++++++++')


def test_f_test():
    data_path = get_data_path('cholesterol.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'trt',
            'data type': 'nominal',
            'categories': ['1time', '2times', '4times', 'drugD', 'drugE']
        },
        {
            'name': 'response',
            'data type': 'ratio'
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': 'trt',
        'dependent variables': 'response',
        'between subjects': 'trt'

    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_path)
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['trt', 'response'])
    # print("\nFrom Field et al.")
    # print("Expected outcome: Oneway ANOVA (F) test")
    print('++++++++++++')


def test_kruskall_wallis():
    data_path = get_data_path('soya.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'Sperm',
            'data type': 'interval'
        },
        {
            'name': 'Soya',
            'data type': 'ordinal',
            'categories': ['No Soya', '1 Soya Meal', '4 Soya Meals', '7 Soya Meals']
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': 'Soya',
        'dependent variables': 'Sperm',
        'between subjects': 'Soya'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_path)
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['Soya', 'Sperm'])

    # print("\nFrom Field et al.")
    # print("Expected outcome: Kruskall Wallis")
    print('++++++++++++')


def test_rm_one_way_anova():
    data_path = get_data_path('co2.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'uptake',
            'data type': 'interval'
        },
        {
            'name': 'Type',
            'data type': 'nominal',
            'categories': ['Quebec', 'Mississippi']
        },
        {
            'name': 'conc',
            'data type': 'ordinal',
            'categories': [95, 175, 250, 350, 500, 675, 1000]
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': ['Type', 'conc'],
        'dependent variables': 'uptake',
        'within subjects': 'conc',
        'between subjects': 'Type'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_path, key="Plant")
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['uptake', 'conc'])

    # print("\nFrom Field et al.")
    # print("Expected outcome: Repeated Measures One Way ANOVA")
    print('++++++++++++')


def test_factorial_anova():
    data_path = get_data_path('gogglesData.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'gender',
            'data type': 'nominal',
            'categories': ['Female', 'Male']
        },
        {
            'name': 'alcohol',
            'data type': 'nominal',
            'categories': ['None', '2 Pints', '4 Pints']
        },
        {
            'name': 'attractiveness',
            'data type': 'interval'
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': ['gender', 'alcohol'],
        'dependent variables': 'attractiveness',
        'between subjects': ['gender', 'alcohol']
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_path)
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['attractiveness', 'gender', 'alcohol'])
    # alcohol main effect?
    # print("\nFrom Field et al.")
    # print("Expected outcome: Factorial ANOVA")
    print('++++++++++++')


def test_two_way_anova():
    data_path = get_data_path('co2.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'uptake',
            'data type': 'interval'
        },
        {
            'name': 'Type',
            'data type': 'nominal',
            'categories': ['Quebec', 'Mississippi']
        },
        {
            'name': 'conc',
            'data type': 'ordinal',
            'categories': [95, 175, 250, 350, 500, 675, 1000]
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': ['Type', 'conc'],
        'dependent variables': 'uptake',
        'within subjects': 'conc',
        'between subjects': 'Type'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
        'groups normally distributed': [['Type', 'uptake'], ['Type', 'conc']],
        'equal variance': [['Type', 'uptake'], ['conc', 'uptake']]
    }

    tea.data(data_path)
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions, mode='relaxed')

    # Fails: not all groups are normal
    tea.hypothesize(['uptake', 'conc', 'Type'])
    # Type main effect?
    # print('Supposed to be 2 way ANOVA')
    print('++++++++++++')


def test_chi_square():
    data_path = get_data_path('catsData.csv')

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'Training',
            'data type': 'nominal',
            'categories': ['Food as Reward', 'Affection as Reward']
        },
        {
            'name': 'Dance',
            'data type': 'nominal',
            'categories': ['Yes', 'No']
        }
    ]
    experimental_design = {
        'study type': 'observational study',
        'contributor variables': 'Training',
        'outcome variables': 'Dance'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_path)
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['Training', 'Dance'])
    # print('Chi square')
    print('++++++++++++')


def test_chi_square_with_dataframe():
    data_path = get_data_path('catsData.csv')

    data_frame = pd.read_csv(data_path)

    # Declare and annotate the variables of interest
    variables = [
        {
            'name': 'Training',
            'data type': 'nominal',
            'categories': ['Food as Reward', 'Affection as Reward']
        },
        {
            'name': 'Dance',
            'data type': 'nominal',
            'categories': ['Yes', 'No']
        }
    ]
    experimental_design = {
        'study type': 'observational study',
        'contributor variables': 'Training',
        'outcome variables': 'Dance'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05,
    }

    tea.data(data_frame)  # Passes data_frame instead of data_path
    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['Training', 'Dance'])
    # print('Chi square')
    print('++++++++++++')



import pandas as pd

import tea
import os

from tea.logging import TeaLoggerConfiguration, TeaLogger
import logging
configuration = TeaLoggerConfiguration()
configuration.logging_level = logging.DEBUG
TeaLogger.initialize_logger(configuration)

# This example is adapted from http://www.real-statistics.com/non-parametric-tests/wilcoxon-rank-sum-test/wilcoxon-rank-sum-exact-test/
def test_mann_whitney_0():
    tea.data('./tests/data/real_stats_3.csv')

    variables = [
        {
            'name': 'Treatment',
            'data type': 'nominal',
            'categories': ['Control', 'Drug']
        },
        {
            'name': 'Score',
            'data type': 'ratio'
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': 'Treatment',
        'dependent variables': 'Score'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05
    }


    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)
    tea.hypothesize(['Treatment', 'Score'], ['Treatment:Control != Drug'])


from tea.ast import NegativeRelationship, PositiveRelationship
import unittest
from unittest.mock import Mock
from tea.build import create_prediction
from parameterized import parameterized


class CreatePredictionTests(unittest.TestCase):

    @parameterized.expand([
        [' ~ +', PositiveRelationship],  # Var_1 ~ +Var_2
        [' ~ ', PositiveRelationship],  # Var_1 ~ Var_2
        [' ~ +', PositiveRelationship, '+'],  # +Var_1 ~ +Var_2
        [' ~ ', PositiveRelationship, '+'],  # +Var_1 ~ +Var_2

        [' ~ -', NegativeRelationship],  # Var_1 ~ -Var_2
        [' ~ ', NegativeRelationship, '-'],  # -Var_1 ~ Var_2
        [' ~ -', NegativeRelationship, '+'],  # +Var_1 ~ -Var_2
        [' ~ +', NegativeRelationship, '-'],  # -Var_1 ~ +Var_2
    ])
    def test_continuous_prediction_should_produce_correct_relationship(self, relation, expected_relationship_type, first_variable_prefix=''):
        prediction_type = 'continuous prediction'
        variable_1_name = 'Var_1'
        variable_2_name = 'Var_2'

        var_mock1 = Mock()
        var_mock1.name = variable_1_name

        var_mock2 = Mock()
        var_mock2.name = variable_2_name

        vars = [var_mock1, var_mock2]
        prediction = f'{first_variable_prefix}{variable_1_name}{relation}{variable_2_name}'
        z = create_prediction(prediction_type, vars, prediction)
        self.assertIsInstance(z, expected_relationship_type)
        self.assertEqual(z.lhs.var, var_mock1)
        self.assertEqual(z.rhs.var, var_mock2)



from tea.helpers.study_type_determiner import StudyTypeDeterminer
from tea.helpers.constants.default_values import DEFAULT_ALPHA_PARAMETER
from tea.runtimeDataStructures.resultData import ResultData
from tea.global_vals import observational_identifier
from tea.runtimeDataStructures.varData import VarData
from tea.ast import Literal, Node, Relate, Variable
from tea.runtimeDataStructures.bivariateData import BivariateData
from tea.runtimeDataStructures.dataset import Dataset
import unittest
from unittest import mock
from unittest.mock import Mock
import pandas as pd
from tea.vardata_factory import VarDataFactory
from unittest.mock import ANY


def create_mocked_study_function(value_to_return=None):
    def mocked_study_function(vars, design):
        return value_to_return
    return mocked_study_function


def create_assign_roles(value_to_return=None):
    def mocked_assign_roles(vars, study_type, design):
        return value_to_return
    return mocked_assign_roles


def create_add_paired_property(value_to_return=None):
    def mocked_add_paired_property(dataset, combined_data, study_type, design):
        return value_to_return
    return mocked_add_paired_property


def create_synthesize_tests(value_to_return=None):
    def mocked_synthesize_tests(dataset, assumptions, combined_data):
        return value_to_return
    return mocked_synthesize_tests


class VarDataFactoryTests(unittest.TestCase):
    def setUp(self):
        self.study_type_determiner_mock = Mock(spec=StudyTypeDeterminer)
        self.varadata_factory = VarDataFactory(self.study_type_determiner_mock)

    def test_vardata_created_for_variable(self):
        dataset = Mock(spec=Dataset)
        mocked_variable_data = {}
        dataset.get_variable_data.return_value = mocked_variable_data
        expression = Mock(spec=Variable)
        expression.name = ''

        # ACT
        returned_value = self.varadata_factory.create_vardata(dataset, expression, {})

        # ASSERT
        self.assertIsInstance(returned_value, VarData)

    def test_vardata_has_correct_metadata_for_variable(self):

        mocked_variable_data = {}
        mocked_expression_name = 'expr-name'

        dataset = Mock(spec=Dataset)
        dataset.get_variable_data.return_value = mocked_variable_data

        expression = Mock(spec=Variable)
        expression.name = mocked_expression_name

        # ACT
        returned_value = self.varadata_factory.create_vardata(dataset, expression, {})

        # ASSERT
        self.assertTrue('var_name' in returned_value.metadata)
        self.assertEqual(returned_value.metadata['var_name'], mocked_expression_name)

    def test_vardata_has_correct_metadata_for_literal(self):
        mocked_expression_value = 'expression value'
        data_for_dataset = [1, 2, 3]
        dataset = Mock(spec=Dataset)

        dataset.data = pd.Series(data_for_dataset)
        expression = Mock(spec=Literal)
        expression.value = mocked_expression_value

        # ACT
        returned_value = self.varadata_factory.create_vardata(dataset, expression, {})

        # ASSERT
        self.assertTrue('value' in returned_value.metadata)
        self.assertEqual(returned_value.metadata['value'], mocked_expression_value)
        self.assertEqual(len(returned_value.properties), len(data_for_dataset))

    def test_should_return_none_for_unknown_node(self):
        dataset = Mock(spec=Dataset)
        expression = Mock(spec=Node)

        # ACT
        returned_value = self.varadata_factory.create_vardata(dataset, expression, {})

        # ASSERT
        self.assertIsNone(returned_value)

    def test_bivariate_analysis_should_use_default_alpha_without_assumptions_called(self):
        data_for_dataset = [1, 2, 3]
        dataset = Mock(spec=Dataset)
        dataset.data = pd.Series(data_for_dataset)

        expression = Mock(spec=Relate)
        expression.vars = []
        expression.predictions = None
        mocked_role_1 = Mock()
        mocked_role_1.role = 'x'
        mocked_role_2 = Mock()
        mocked_role_2.role = 'x'

        result_data_mock = Mock(spec=ResultData)
        self.study_type_determiner_mock.determine_study_type = create_mocked_study_function(observational_identifier)

        with mock.patch('tea.vardata_factory.synthesize_tests', side_effect=create_synthesize_tests([])), \
                mock.patch('tea.vardata_factory.assign_roles', side_effect=create_assign_roles([mocked_role_1, mocked_role_2])), \
                mock.patch('tea.vardata_factory.ResultData', side_effect=lambda x, y: result_data_mock),\
                mock.patch('tea.vardata_factory.execute_test', side_effects=lambda x, y, z, a, b: []),\
                mock.patch('tea.vardata_factory.add_paired_property', side_effect=create_add_paired_property(None)) as mocked_add_paired_property:
            self.varadata_factory.create_vardata(dataset, expression, {})

            mocked_add_paired_property.assert_called_with(ANY, ANY, ANY, ANY)
            called_with = {type(arg): arg for arg in mocked_add_paired_property.call_args[0]}
            self.assertTrue(BivariateData in called_with)
            self.assertEqual(DEFAULT_ALPHA_PARAMETER, called_with[BivariateData].alpha)



import pandas as pd

import tea
import os

from tea.logging import TeaLoggerConfiguration, TeaLogger
import logging
configuration = TeaLoggerConfiguration()
configuration.logging_level = logging.DEBUG
TeaLogger.initialize_logger(configuration)


base_url = 'https://homes.cs.washington.edu/~emjun/tea-lang/datasets/'
file_names = ['UScrime.csv', 'statex77.csv', 'catsData.csv', 'cholesterol.csv', 'soya.csv', 'co2.csv', 'exam.csv', 'liar.csv',
              'pbcorr.csv', 'spiderLong_within.csv', 'drug.csv', 'alcohol.csv', 'ecstasy.csv', 'gogglesData.csv', 'gogglesData_dummy.csv']
data_paths = [None] * len(file_names)


def load_data():
    global base_url, data_paths, file_names
    global drug_path

    for i in range(len(data_paths)):
        csv_name = file_names[i]

        csv_url = os.path.join(base_url, csv_name)
        data_paths[i] = tea.download_data(csv_url, csv_name)


def get_data_path(filename):
    load_data()
    try:
        data_idx = file_names.index(filename)
    except:
        raise ValueError(f"File is not found!:{filename}")
    data_path = data_paths[data_idx]

    return data_path

# This example is adapted from http://www.real-statistics.com/non-parametric-tests/wilcoxon-signed-ranks-test/
def test_wilcoxon_signed_rank_0():
    tea.data('./tests/data/real_stats_0.csv', key='Person')

    variables = [
        {
            'name': 'Person',
            'data type': 'ratio'
        },
        {
            'name': 'Side',
            'data type': 'nominal',
            'categories': ['Right', 'Left']
        },
        {
            'name': 'Score',
            'data type': 'ratio'
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': 'Side',
        'dependent variables': 'Score',
        'within subjects': 'Side',
        # 'key': 'Person'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05
    }


    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['Side', 'Score'], ['Side:Right != Left'])


# This example is adapted from Example 1 on http://www.real-statistics.com/non-parametric-tests/wilcoxon-signed-ranks-test/
def test_wilcoxon_signed_rank_1():
    tea.data('./tests/data/real_stats_1.csv')

    variables = [
        {
            'name': 'Subject',
            'data type': 'ratio'
        },
        {
            'name': 'Source',
            'data type': 'nominal',
            'categories': ['Memory', 'Median']
        },
        {
            'name': 'Score',
            'data type': 'ratio'
        }
    ]
    experimental_design = {
        'study type': 'experiment',
        'independent variables': 'Source',
        'dependent variables': 'Score',
        'within subjects': 'Source'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05
    }


    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['Source', 'Score'], ['Source:Memory != Median'])


# This example is adapted from Example 2 on  http://www.real-statistics.com/non-parametric-tests/wilcoxon-signed-ranks-test/wilcoxon-signed-ranks-exact-test/
def test_wilcoxon_signed_rank_2():
    tea.data('./tests/data/real_stats_2.csv', key='Couple')

    variables = [
        {
            'name': 'Couple',
            'data type': 'ratio'
        },
        {
            'name': 'Person',
            'data type': 'nominal',
            'categories': ['Wife', 'Husband']
        },
        {
            'name': 'Score',
            'data type': 'ratio'
        }
    ]
    experimental_design = {
        'study type': 'observational study',
        'contributor variables': 'Person',
        'outcome variables': 'Score',
        'within subjects': 'Person'
    }
    assumptions = {
        'Type I (False Positive) Error Rate': 0.05
    }


    tea.define_variables(variables)
    # Allows for using multiple study designs for the same dataset (could lead to phishing but also practical for saving analyses and reusing as many parts of analyses as possible)
    tea.define_study_design(experimental_design)
    tea.assume(assumptions)

    tea.hypothesize(['Person', 'Score'], ['Person:Wife != Husband'])


    ## TODO: one-sided!


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(name='tealang',
      version='0.3.1',
      author='Eunice Jun',
      author_email='emjun@cs.washington.edu',
      description='Tea: A High-level Language and Runtime System to Automate Statistical Analysis',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/emjun/tea-lang',
      packages=setuptools.find_packages(),
      install_requires=[
          'attrs',
          'pandas',
          'scipy',
          'scikit-learn',
          'statsmodels',
          'bootstrapped',
          'pipfile',
          'requests',
          'z3-solver',
          'urllib3',
          'parameterized', 
          'sympy'
      ],
      classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)


import tea

variables = [
    {
        'name': 'Checked',
        'data type': 'nominal',
        'categories': ['not checked', 'checked']
    },
    {
        'name': 'Affiliation Uni',
        'data type': 'nominal',
        'categories': ['0', '1']
    }
]

study_design = {
    'study type': 'observational study',
    'contributor variables': ['Affiliation Uni'],
    'outcome variables': 'Checked',
}

assumptions = {'alpha': 0.05}


tea.data("./tests/data/v3_data.csv")
tea.define_variables(variables)
tea.define_study_design(study_design)
tea.assume(assumptions)
tea.hypothesize(["Affiliation Uni", "Checked"], ["Affiliation Uni:1 > 0"])
# tea.define_study_design(study_design)
# tea.assume(assumptions)
# tea.hypothesize(['Affiliation Uni', 'Checked'], ['Affiliation Uni:1 > 0'])